function con2dis(action)

% Last Update: 20-Oct-2001
%   : Revision history contained in readme.m file

% This GUI was written by Gregory Krudysz during Summer/Fall 2001. 
%
% Acknowledgement: 
%  I would like to thank Jordan Rosenthal and Prof. McClellan for their support
%  and encouragement in writing this GUI.  Some of the files were written by 
% Jordan and/or modified by Prof. McClellan.
%
% Gregory Krudysz, Fall 2001
%-----------------------------------------------------------------------------

if nargin == 0
   action = 'Initialize';
else
   h =getuprop(gcbf,'Handles'); 	%getappdata(gcbf, 'DataStructure');
end


%globals
Ver = 'con2dis Version 0.98';  % Version string for figure title
Xlimit = 0.50;		 % x-axis limits for Axes 1 and 2
YSPMAX = 1.05;     % max height of the spectrum display
xMAX = 1.05;
%----------------------

switch action
   %-------------------------------------------------------------------------------
case 'Initialize'
   %-------------------------------------------------------------------------------
   
   %---  Check the installation, the Matlab Version, and the Screen Size  ---%
   errCmd = 'errordlg(lasterr,''Error Initializing Figure'');error(lasterr);';
   cmdCheck1 = 'installcheck;';
   cmdCheck2 = 'h.MATLABVER = versioncheck(5.2);';
   
   cmdCheck3 = 'screensizecheck([800 600]);';
   cmdCheck4 = ['adjustpath(''' mfilename ''');'];
   eval(cmdCheck1,errCmd);       % Simple installation check
   eval(cmdCheck2,errCmd);       % Check Matlab Version
   eval(cmdCheck3,errCmd);       % Check Screen Size
   eval(cmdCheck4,errCmd);       % Adjust path if necessary
  
  % GUI
  ver = version;
   h.MATLABVER =str2num(ver(1:3));
  if h.MATLABVER < 5.3
	  gui2
  else
	  gui
  end

   set(gcf, 'Name',Ver);
   h.FigPos = get(gcf,'Pos');
   
   SCALE = getfontscale;          % Platform dependent code to determine SCALE parameter
   setfonts(gcf,SCALE);           % Setup fonts: override default fonts used in con2dis
   h = gethandles(h);             % Get GUI graphic handles
   h = defaultplots(h,Ver);       % Create default plots
   configresize(gcf);             % Change all 'units'/'font units' to normalized

   %---  Store the structure of handles/data for use later during a callback  ---%
   %setappdata(gcf, 'DataStructure', h);
   setuprop(gcf, 'Handles', h);   % Store handles as new userproperty


%====================================================================
%====================================================================
%==================== END OF ========================================
%================== INITIALIZE ======================================
%====================================================================
%====================================================================
%====================================================================
%====================================================================
%====================================================================
case 'SetFigSize'
      
   set(gcbf, 'Units', 'Normalized');
   FigSize = [0.10 0.10 0.55 0.80];
   set(gcbf, 'Position', FigSize);
   
%====================================================================
  case 'ResizeFcn'
%====================================================================
% Fix for bugs in normalized fontunits in Matlab 5.2.
% Force constant figure aspect ratio if in Matlab 5.3.
h.FigPos = get(gcf,'Pos');
ver = version;
h.MATLABVER =str2num(ver(1:3));

FigPos = resizefcn(h.FigPos,gcbo,h.MATLABVER);  % Version dependent resize code
	switch computer
	case 'MAC2'
		% On MAC, baseline of text inside edit boxes remains at same
		% vertical position irregardless of change in font size.
		% To properly align text, here the old edit boxes are deleted
		% and new ones created with the proper size.
		hEd = findall(gcbf,'type','uicontrol','style','edit');
		OldFontUnits = get(hEd,'FontUnits');
		set(hEd,'FontUnits','Pixels');
		hEdNew = zeros(size(hEd));
		relHeightChange = FigPos(4)/h.FigPos(4);
		for i = 1:length(hEd)
			Props = get(hEd(i));
			FontSize = relHeightChange*Props.FontSize;
			Props = rmfield(Props,{'Extent','Type','FontSize','FontUnits'});
			delete(hEd(i));
			hEdNew(i) = uicontrol('FontUnits','Pixels', ...
			'FontSize',FontSize,Props);
		end
      set(hEdNew,{'FontUnits'},OldFontUnits);
      h = gethandles(h);
		setuprop(gcbf,'Handles',h);
	end

	sethandles(h,'FigPos',FigPos);                  % Store old position

%====================================================================
case 'Editbox1'
%====================================================================   

Fo = get(h.Editbox1,'String');
   FoN = str2num(Fo);
   FoMAX = get(h.Slider1,'Max');
   FoN = min([FoN,FoMAX]);
   FsN = get(h.Slider2,'Value');
   
   if h.wflag == 1
      pi2 = 2*pi; 
      FoN = FoN/pi2;
      FsN = FsN/pi2;
      Fo = sprintf('%.1f',FoN);
   end
   
   Fochange(h,FoN,FsN,h.time1);    

%====================================================================
case 'Editbox2'
%===================================================================
  
Fs = get(h.Editbox2,'String');
   FsN = str2num(Fs);
   FoN = get(h.Slider1,'Value');
   MaxFs = get(h.Slider2,'Max');
   MinFs = get(h.Slider2,'Min');
   
   
   if h.wflag == 1
      pi2 = 2*pi; 
      FoN = FoN/pi2;
      FsN = FsN/pi2;
      MaxFs = MaxFs/pi2;
      MinFs = MinFs/pi2;
   end
      
   FsN = min( [max([MinFs,FsN]),MaxFs] );
   
   if h.wflag == 1
       WsS = sprintf('%.1f',pi2*FsN);
       set(h.Editbox2,'String',WsS)
   else   
       FsS = sprintf('%.1f',FsN);
       set(h.Editbox2,'String',FsS)
   end
   
   if FoN>1.49*FsN
       FoN = floor(1.49*FsN*10)/10;
   end
   
   P = pi;
   R = FoN/FsN;
   if (abs(mod(R,1))<1e-3)
       Xsp = abs(cos(h.phaseN));
   else
       Xsp = 0.5;
   end

   Fochange(h,FoN,FsN,h.time1)
   
   if h.wflag == 1
              
       Fo = sprintf('%.1f',pi2*FoN);
       set(h.Editbox1,'String',Fo);
       set(h.Slider1,'Value',pi2*FoN);
       set(h.Slider1,'Max',1.49*pi2*FsN);
       set(h.Slider2,'Value',pi2*FsN);   
       set(h.TextFs,'String',sprintf('\\omega_s = %.1f (rad/s)',pi2*FsN));
       
   else
              
       Fo = sprintf('%.1f',FoN);
       set(h.Editbox1,'String',Fo);
       set(h.Slider1,'Value',FoN);
       set(h.Slider1,'Max',1.49*FsN);
       set(h.Slider2,'Value',FsN);
       set(h.TextFs,'String',sprintf('f_s = %.1f (Hz)',FsN));
   end
   
   % Axes2 ----------------
   FoNc2 = phaseText(h,FoN,FsN,R);
   time2 = 0:1/FsN:0.5;
   stemdata(time2,cos(2*pi*FoNc2*time2 + h.phaseN),h.Line2);
   
   if length(time2) > 12
      time2a = 0:2/FsN:1;
      set(h.Axis2,'XTick',time2a ...
                 ,'XTickLabel',2*(0:length(time2a)-1)');
   else
      set(h.Axis2,'XTick',time2 ...
                 ,'XTickLabel',(0:length(time2)-1)');
   end
   
   %Axes4 --------------------
   if (abs(FoN)<1e-3)
       XCsp = abs(cos(h.phaseN));
   else
       XCsp = 0.5;
   end
   stemdata([FoN],XCsp,h.Line4);
   stemdata([-FoN],XCsp,h.Line4c);
   set(h.Line4d,'XData',[-FsN -FsN nan 0 0 nan FsN FsN] ...
       ,'YData',[0 YSPMAX nan 0 YSPMAX nan 0 YSPMAX]);
   if  h.wflag
       set([h.Axis4,h.Axis6],'XLim',[-1.5*FsN 1.5*FsN] ...
           ,'XTick',FsN*(-1:0.5:1) ...
           ,'XTickLabel',...
           {sprintf('%.1f',-pi2*FsN), '', '0',...
               '', sprintf('%.1f',pi2*FsN)} );
   else
       set([h.Axis4,h.Axis6],'XLim',[-1.5*FsN 1.5*FsN] ...
           ,'XTick',FsN*(-1:0.5:1) ...
           ,'XTickLabel',...
           {sprintf('%.1f',-FsN), '', '0',...
               '', sprintf('%.1f',FsN)} );
   end
   set([h.Patch4,h.Patch6],'XData',[-FsN/2 FsN/2 FsN/2 -FsN/2] ...
                          ,'YData',[0 0 YSPMAX YSPMAX]);         
   % Axes3 -------------------
   stemdata([-2+R -1+R 1+R  2+R],Xsp*[1,1,1,1],h.Line3out);
   stemdata([-2-R -1-R 1-R  2-R],Xsp*[1,1,1,1],h.Line3outC);
   stemdata([R],Xsp,h.Line3in);       
   stemdata([-R],Xsp,h.Line3inC);       
   
   % Axes6 --------------------
   if FoN > FsN/2
       set([h.Line6,h.Line6c],'Color','r');
       set(h.TextAlie,'Visible','On');
       set(h.Axis6,'Color', 0.75*[1,1,1]);
   elseif FoN == FsN/2
       set([h.Line6,h.Line6c],'Color','m');
       set(h.Axis6,'Color', 0.9*[1,1,1]);
       set(h.TextAlie,'Visible','Off');
   else
       set([h.Line6,h.Line6c],'Color','b');
       set(h.TextAlie,'Visible','Off');
       set(h.Axis6,'Color','w');
   end
   
%====================================================================
case 'Editbox3'
%====================================================================   
   FoN = get(h.Slider1,'Value');
   FsN = get(h.Slider2,'Value');
   R = FoN/FsN;
   P = pi; 
   
   if h.wflag == 1
      FoN = FoN/(2*P);
      FsN = FsN/(2*P);
   end
   
   phase = get(h.Editbox3,'String');
   h.phaseN = str2num(phase);
   setappdata(gcf, 'DataStructure', h);
   set(h.TextPh,'String',sprintf('Phase = %.2f',h.phaseN));
   
%   FoNc2 = phaseText (h,FoN,FsN,R);

   Fochange(h,FoN,FsN,h.time1);
   
if 0
% Axes1
   time2 = 0:1/FsN:1;
   c1 = cos(2*P*FoN*h.time1 + h.phaseN);
   c2 = cos(2*P*FoNc2*time2 + h.phaseN);
   set([h.Line1; h.Line11],'YData',c1);
 
   % Axes2
   stemdata(time2,c2,h.Line2);
end

%====================================================================
case 'Slider1'
%====================================================================   
   FoN = get(h.Slider1,'Value');
   
   numlim = 1e-3;         % prevent extremely small period T
   if FoN < numlim
      set(h.Slider1,'Value',numlim);
      FoN = numlim;
   end
   
   if h.wflag == 1
      Fo = sprintf('%.1f',FoN);
   else
   	Fo = sprintf('%.1f',FoN); 
   end
   
   set(h.Editbox1,'String',Fo);
   
   con2dis Editbox1
           
%====================================================================
case 'Slider2'    
   FsN = get(h.Slider2,'Value');
%====================================================================   
   if h.wflag == 1
      FsN = FsN/(2*pi);
   end
   
   numlim = 1e-3;         % prevent extremly small period T
   if FsN < numlim
      
      if h.wflag == 1
         set(h.Slider2,'Value',2*pi*numlim);
      else
      	set(h.Slider2,'Value',numlim);
      end
      
      FsN = numlim;
      
   else
      if h.wflag == 1
         set(h.Slider2,'Value',2*pi*FsN);
      else
      	set(h.Slider2,'Value',FsN);
      end

   end
   
   
   if h.wflag == 1
      Fs = sprintf('%.1f',2*pi*FsN);   
   else
   	Fs = sprintf('%.1f',FsN); 
   end
   
   set(h.Editbox2,'String',Fs);
   
   con2dis Editbox2
  
%====================================================================
case 'SetLineWidth'
%====================================================================
   newLineWidth = linewidthdlg(h.LineWidth);
   h.MarkerSize = h.MarkerSize + (newLineWidth - h.LineWidth);
   h.LineWidth = newLineWidth;
   set(findobj(gcbf,'type','line'),'linewidth',h.LineWidth);
   setappdata(gcbf, 'DataStructure', h);
   
%====================================================================
case 'LineDragStart'
%====================================================================  
   setptr(gcbf, 'closedhand');
   CurrentPoint = get(gca, 'CurrentPoint');
   setappdata(gcbf, 'CurrentX', CurrentPoint(1,1) );
   set(gcbf,'WindowButtonUpFcn',sprintf('%s LineDragStop',mfilename));
   set(gcbf,'WindowButtonMotionFcn',[mfilename ' MoveLine']);
   
%====================================================================
case 'MoveLine'    
%====================================================================
   CurrentPoint = get(gca, 'CurrentPoint');
   CurrentX = CurrentPoint(1,1);
   
   FoN = CurrentX*( 2*(CurrentX>0)-1 );
   FoN = round(10*FoN)/10;   %- Round to nearest tenth
   %if CurrentX < 0
   %   FoN = - CurrentX;
   %else
   %   FoN = CurrentX;   
   %end
   
   Fo = sprintf('%.1f',FoN);
   
   FsN = get(h.Slider2,'Value');
   
   if h.wflag == 1   
      FsN = FsN/(2*pi);
   end
   
   Fochange(h,FoN,FsN,h.time1);
   
   setappdata(gcbf, 'CurrentX', CurrentX);
    % PROBLEM setuprop(gcf, 'Handles', h);
%====================================================================
case 'LineDragStop'
%====================================================================
   
   rmappdata(gcbf,'CurrentX');
   set(gcbf, 'WindowButtonMotionFcn', '');
   set(gcbf, 'WindowButtonUpFcn', '');
   setptr(gcbf,'arrow');
   
%====================================================================
case 'PlotsMenu'   
%====================================================================   
   check = get(h.AllMenu,'Checked');
   if strcmp(check,'off')
      con2dis Unfold
   else
      con2dis Foldin;
   end
      
%====================================================================
case 'Unfold'
%====================================================================   
   set(gcbo,'Checked','on')
   set(h.Group,'Units','Characters');
   FigSize = get(gcbf,'Position');
   FigSizeNew = [1 1 1.5 1].* FigSize;
   set(gcbf, 'Position', FigSizeNew);
   set(h.Group,'Units','normalized');
  
   set(h.Hide,'Visible','On');
   set([h.Line6,h.Line6c],'Visible','On');
   
%====================================================================
case 'Foldin'
%====================================================================   
   set(gcbo,'Checked','off');
   set(h.Group,'Units','Characters');
   FigSize = get(gcbf,'Position');
   FigSizeNew = [1 1 1/1.5 1].*FigSize;
   set(gcbf,'Position', FigSizeNew);
   set(h.Hide,'Visible','Off');
   set([h.Line6,h.Line6c],'Visible','Off');
   set(h.Group,'Units','Normalized');
   
%====================================================================
case 'ShowW' 
%====================================================================
   show = get(h.ShowW,'Checked');
   pi2 = 2*pi;
   
   if strcmp(show,'off')
      set(h.ShowW,'Checked','on'); 
      % change text labels
      set(gcf,'CurrentAxes',h.Axis4);
      xlabel('\omega (rad/s)')
      set(gcf,'CurrentAxes',h.Axis6);
      xlabel('\omega (rad/s)')
      
      % Find and change Fo and Fs
      roundoff = 0.6;
      Fo = get(h.Editbox1,'String');
      FoN = get(h.Slider1,'Value');
      FsN = get(h.Slider2,'Value');
      WoN = pi2*FoN;
      WsN = pi2*FsN;
      Wo = sprintf('%.1f',WoN);
      Ws = sprintf('%.1f',WsN);
      MaxS2 = get(h.Slider2,'Max');
      MaxS2d = pi2*MaxS2;
      MinS2 = get(h.Slider2,'Min');
      MinS2d = pi2*MinS2;
      MaxS1d = pi2*1.49*FsN;
      R = FoN/FsN;
      
      set(h.TextFo,'String',sprintf('\\omega_o = %.1f (rad/s)',WoN));
      set(h.TextFs,'String',sprintf('\\omega_s = %.1f (rad/s)',WsN));

      set(h.Editbox1,'String',Wo);
      set(h.Editbox2,'String',Ws);
      set(h.Slider1,'Min',0,'Max',MaxS1d,'Value',WoN);
      set(h.Slider2,'Min',MinS2d,'Max',MaxS2d,'Value',WsN);        
      set([h.Axis4,h.Axis6],...
          'XTickLabel',{sprintf('%.2f',-pi2*FsN),'','0','',sprintf('%.2f',pi2*FsN)} );        

      set(h.RButton2,'Value',1);
      h.wflag = 1;
      setappdata(gcf, 'DataStructure', h);
      phaseText(h,FoN,FsN,R);
        
else 
      set(h.ShowW,'Checked','off');
      % change text labels
      set(gcf,'CurrentAxes',h.Axis4);
      xlabel('f (Hz)')
      set(gcf,'CurrentAxes',h.Axis6);
      xlabel('f (Hz)')
      
      % Find and change Fo and Fs
      Wo = get(h.Editbox1,'String');
      WoN = get(h.Slider1,'Value');
      WsN = get(h.Slider2,'Value');
      FoN = WoN/pi2;  
      FsN = WsN/pi2;
      Fo = sprintf('%.1f',FoN);
      Fs = sprintf('%.1f',FsN);
      MaxS2 = get(h.Slider2,'Max');
      MaxS2h = MaxS2/pi2;
      MinS2 = get(h.Slider2,'Min');
      MinS2h = MinS2/pi2;
      MaxS1h = 1.49*FsN;
      R = FoN/FsN;
      
      set(h.TextFo,'String',sprintf('f_o = %.1f (Hz)',FoN));
      set(h.TextFs,'String',sprintf('f_s = %.1f (Hz)',FsN));
      
      set(h.Editbox1,'String',Fo);
      set(h.Editbox2,'String',Fs);
      set(h.Slider1,'Min',0,'Max',MaxS1h,'Value',FoN);
      set(h.Slider2,'Min',MinS2h,'Max',MaxS2h,'Value',FsN);       
      set([h.Axis4,h.Axis6],'XTickLabel',FsN*(-1:0.5:1) );         

      set(h.RButton2,'Value',0);
      h.wflag = 0;
      setappdata(gcf, 'DataStructure', h);
      phaseText(h,FoN,FsN,R);
     
end

%******************************************************************
case 'ShowW_hat' 
%====================================================================
   show_hat = get(h.ShowW_hat,'Checked');
   pi2 = 2*pi;
  
   if strcmp(show_hat,'off')
      set(h.ShowW_hat,'Checked','on'); 
      % change text labels
      h.FontSizeSym = get(h.Axis3,'FontSize');
      set(gcf,'CurrentAxes',h.Axis3);
      %xlabel('{}_{^{\fontsize{11}\omega = 2\pi (f_o / f_s)}}^{\^}') 
      
      xlabel('\omega = 2\pi (f_o / f_s)')
      %set(h.hat,'position',[-0.24 -0.12]);
      
      set(h.Axis3,'FontName','Symbol'...
             ,'XTick',0.5*(-2:2),'XTickLabel',{'-2p';'-p';'0';'p';'2p'}...
             ,'FontWeight','bold','FontSize',h.FontSizeSym)                  
         
      set(h.RButton22,'Value',1);
      h.w_hat_flag = 1;
      setappdata(gcf, 'DataStructure', h);
else 
   set(h.ShowW_hat,'Checked','off');
    h.FontSize = get(h.Axis3,'FontSize');
      % change text labels
      set(gcf,'CurrentAxes',h.Axis3);
      set(h.Axis3,'FontName','Helvetica',...
         'XTickLabel',{'-1';'-0.5';'0';'0.5';'1'},...
         'FontSize',h.FontSize);  
 
      %xlabel('{}_{^{\fontsize{11} f = f_o / f_s}}^{\^}')
      xlabel('f = f_o / f_s')
      %set(h.hat,'position',[-0.21 -0.12]);
      
      set(h.RButton22,'Value',0);
      h.w_hat_flag = 0;
      setappdata(gcf, 'DataStructure', h);
end

%====================================================================
case 'showLF'
%====================================================================
   check = get(h.ShowLF,'Checked');
   if strcmp(check,'off')
      set(h.ShowLF,'Checked','on');
      set(h.Line11,'Visible','off');
      set(gcf,'CurrentAxes',h.Axis2);
      set(h.Line52,'Visible','on');
      set(h.RButton1,'Value',1);
   else
      set(h.ShowLF,'Checked','off');
      set(h.Line11,'Visible','on');
      set(h.Line52,'Visible','off');
      set(h.RButton1,'Value',0);
   end

%====================================================================
case 'FontResize' % This is not finished yet
%====================================================================
   CurrentFontSizeT = get(h.TextFs,'FontSize');
   CurrentFontSizeA = get(h.Axis1,'FontSize'); %font units get here 'normalized'
   CurrentFontWtA = get(h.Axis1,'FontWeight');
   CurrentFontWtT = get(h.TextFs,'FontWeight');
   
   if strcmp(CurrentFontWtA,'bold')
      CurrentFontWtA = 1;
   else
      CurrentFontWtA = 0;
   end
   
   if strcmp(CurrentFontWtT,'bold')
      CurrentFontWtT = 1;
   else
      CurrentFontWtT = 0;
   end
   Vec = [CurrentFontSizeT CurrentFontWtT CurrentFontSizeA CurrentFontWtA];
   set(0,'UserData',Vec);
   [FontTnew,BoldTnew,FontAnew,BoldAnew] = axesfontdlg
   
   %set(0,'UserData',[AxesFontSize Boldit]);
   
   if BoldAnew == 0;
      FontWeightA = 'normal';
   else
      FontWeightA = 'bold';
   end
   
   if BoldTnew == 0;
      FontWeightT = 'normal';
   else
      FontWeightT = 'bold';
   end
   
   set(findobj(gcbf,'type','axes'),'FontSize',FontAnew, ...
                                   'FontWeight',FontWeightA);
   set(findobj(gcbf,'type','text'),'FontSize',FontTnew, ...
                                   'FontWeight',FontWeightT);
                                                             
%====================================================================
case 'Button1'
%====================================================================
   
valB1 = get(h.RButton1,'Value');
	if valB1 == 1
      set(h.RButton1,'Value',1); 
      set(h.ShowLF,'Checked','Off');   
	else
   	set(h.RButton1,'Value',0);      
   	set(h.ShowLF,'Checked','On');   
	end
        
con2dis showLF
   
%====================================================================
case 'Button2'
%====================================================================
     
valB2 = get(h.RButton2,'Value');

	if valB2 == 1
   	set(h.RButton2,'Value',1); 
      set(h.ShowW,'Checked','off');   
	else
   	set(h.RButton2,'Value',0); 
   	set(h.ShowW,'Checked','on');   
	end    

con2dis ShowW
     
%====================================================================
case 'Button22'
%====================================================================
     
valB2 = get(h.RButton22,'Value');

	if valB2 == 1
   	set(h.RButton22,'Value',1); 
      set(h.ShowW_hat,'Checked','off');   
	else
   	set(h.RButton22,'Value',0); 
   	set(h.ShowW_hat,'Checked','on');   
	end    

con2dis ShowW_hat
     
%====================================================================

otherwise
 
   error('Illegal action');
   
end

%--------------------------------------------------------------------------------------
% SETHANDLES
%--------------------------------------------------------------------------------------
function Handles = sethandles(Handles,field,value)
%SETHANDLES
%   Handles = sethandles(Handles,field,value) sets the field to the 
%   given value within the structure Handles and then saves the
%   structure in the current figure using SETUPROP.
%
%   See also SETUPROP
%
% Jordan Rosenthal

Handles = setfield(Handles,field,value);
setuprop(gcbf, 'Handles', Handles);
